use std::env;
use std::fs;
use std::fs::File;
use std::io::{BufWriter, Write};
use std::process::ExitCode;

const SECTOR_SIZE: usize = 96;
const CHANNEL_SIZE: usize = SECTOR_SIZE / 8;
const LEADIN_SKIP_SECTORS: usize = (10 * 60 + 2) * 75;

fn main() -> ExitCode {
    let args: Vec<_> = env::args_os().collect();
    if args.len() != 3 {
        println!("usage: redumper-sub in.subcode out.sub");
        return ExitCode::FAILURE;
    }
    let infile = fs::read(&args[1]).expect("read input");
    assert!(infile.len() % SECTOR_SIZE == 0);
    let mut outfile = BufWriter::new(File::create(&args[2]).expect("open output"));

    for sector in infile.chunks(SECTOR_SIZE).skip(LEADIN_SKIP_SECTORS) {
        for channel_bit in [7, 6, 5, 4, 3, 2, 1, 0] {
            let out_channel: [u8; CHANNEL_SIZE] = std::array::from_fn(|ch_i| {
                let mut b_out = 0;

                for b_i in 0..8 {
                    let bit = ((sector[ch_i * 8 + b_i]) >> channel_bit) & 1;
                    b_out |= bit << (7 - b_i);
                }

                b_out
            });
            outfile.write_all(&out_channel).expect("write output");
        }
    }

    outfile.flush().expect("flush output");

    ExitCode::SUCCESS
}
